import * as i0 from '@angular/core';
import { EventEmitter, Component, Input, Output, NgModule } from '@angular/core';
import * as d3 from 'd3';
import * as i1 from 'ng-container-terminal/tool';
import { CommonModule } from '@angular/common';

class CtYardBayComponent {
    constructor(el, yardposParser) {
        this.el = el;
        this.yardposParser = yardposParser;
        this.poses = [];
        this.padding = 16;
        this._size = { row: 6, tier: 5 };
        this.displaySize = { row: 6, tier: 5 };
        this.cellSize = 16;
        this.yardposClick = new EventEmitter();
    }
    set size(s) {
        this._size = { row: 6, tier: 5 };
        this.updateYardposInfo();
        // this.resize();
    }
    get size() {
        return this._size;
    }
    set renderOptions(options) {
        this._renderOptions = options;
        // this.renderLayout();
        // this.updateYardposInfo();
        setTimeout(() => {
            this.renderLayout();
            this.updateYardposInfo();
        }, 0);
    }
    get renderOptions() {
        return this._renderOptions;
    }
    set yardBay(yardBay) {
        this._yardBay = yardBay;
        this.updateYardposInfo();
    }
    get yardBay() {
        return this._yardBay;
    }
    ngOnInit() {
        // if (this.yardBay.dataUpdated) {
        //   this.yardBay.dataUpdated.subscribe(() => {
        //     this.updateYardposInfo();
        //   });
        // }
        this.host = d3.select(this.el.nativeElement);
        this.svg = this.host
            .select('svg')
            .attr('width', (this.displaySize.row + 1) * this.cellSize + 2 * this.padding)
            .attr('height', (this.displaySize.tier + 1) * this.cellSize + 2 * this.padding);
        this.yardBayGroup = this.svg.append('g').attr('class', 'yard-bay-group');
        this.yardposInfoGroup = this.svg.append('g').attr('class', 'yard-pos-info-group');
        this.renderLayout();
        this.updateYardposInfo();
    }
    generatePoses() {
        const posMap = {};
        this.yardBay.poses.forEach(p => {
            posMap[p.name] = p;
        });
        const poses = [];
        for (let row = 1; row <= this.displaySize.row; row++) {
            for (let tier = 1; tier <= this.displaySize.tier; tier++) {
                const posName = this.yardBay.name + this.yardposParser.formatP(row) + this.yardposParser.formatC(tier);
                if (posMap[posName]) {
                    poses.push(posMap[posName]);
                }
                else {
                    poses.push({
                        name: this.yardBay.name + this.yardposParser.formatP(row) + this.yardposParser.formatC(tier),
                        data: null
                    });
                }
            }
        }
        this.poses = poses;
    }
    resize() {
        let maxRow, maxTier = 0;
        this.yardBay.poses.forEach(pos => {
            const row = +this.yardposParser.getP(pos.name);
            const tier = +this.yardposParser.getC(pos.name);
            if (row > maxRow) {
                maxRow = row;
            }
            if (tier > maxTier) {
                maxTier = tier;
            }
        });
        this.displaySize = {
            row: Math.max(this.displaySize.row, maxRow),
            tier: Math.max(this.displaySize.tier, maxTier)
        };
        // this.yardBay.poses.;
    }
    ngOnChanges(changes) {
        if (changes['yardBay']) {
            setTimeout(() => {
                // console.log(this.yardBay);
                this.host = d3.select(this.el.nativeElement);
                this.svg = this.host
                    .select('svg')
                    .attr('width', (this.displaySize.row + 1) * this.cellSize + 2 * this.padding)
                    .attr('height', (this.displaySize.tier + 1) * this.cellSize + 2 * this.padding);
                this.renderLayout();
                this.updateYardposInfo();
            }, 0);
        }
    }
    /**
     * 绘制贝位结构
     */
    renderLayout() {
        // 绘制层标签
        this.svg.selectAll('g.yard-bay-tier-label-group').remove();
        this.yardBayTierLabelGroup = this.svg.append('g').attr('class', 'yard-bay-tier-label-group');
        this.yardBayTierLabelGroup
            .selectAll('text')
            .data(d3.range(1, this.displaySize.row + 1))
            .enter()
            .append('text')
            .attr('width', this.cellSize)
            .attr('height', this.cellSize)
            .attr('transform', label => {
            const x = 0;
            const y = (this.displaySize.tier - label) * this.cellSize;
            return `translate(${x}, ${y})`;
        })
            .attr('font-size', '12')
            .attr('text-anchor', 'middle')
            .attr('dx', (this.cellSize / 3) * 2)
            .attr('dy', (this.cellSize / 3) * 2)
            .text(c => c);
        // 绘制列标签
        this.svg.selectAll('g.yard-bay-row-label-group').remove();
        this.yardBayRowLabelGroup = this.svg.append('g').attr('class', 'yard-bay-row-label-group');
        this.yardBayRowLabelGroup
            .selectAll('text')
            .data(d3.range(1, this.displaySize.row + 1))
            .enter()
            .append('text')
            .attr('width', this.cellSize)
            .attr('height', this.cellSize)
            .attr('transform', label => {
            const x = label * this.cellSize;
            const y = this.displaySize.tier * this.cellSize;
            return `translate(${x}, ${y})`;
        })
            .attr('font-size', '12')
            .attr('text-anchor', 'middle')
            .attr('dx', this.cellSize / 2)
            .attr('dy', this.cellSize / 2)
            .text(c => c);
        // 绘制区位号标签
        this.svg.selectAll('g.yard-bay-name-label-group').remove();
        this.yardBayNameLabelGroup = this.svg.append('g').attr('class', 'yard-bay-name-label-group');
        this.yardBayNameLabelGroup
            .append('text')
            .attr('transform', label => {
            const x = ((this.displaySize.row + 1) / 2.0) * this.cellSize;
            const y = (this.displaySize.tier + 1) * this.cellSize;
            return `translate(${x}, ${y})`;
        })
            .attr('font-size', '16')
            .attr('text-anchor', 'middle')
            .attr('dx', this.cellSize / 2)
            .attr('dy', this.cellSize / 2)
            .text(this.yardBay.name);
    }
    updateYardposInfo() {
        // console.log('updating....');
        this.generatePoses();
        if (!this.yardposInfoGroup) {
            return;
        }
        const cell = this.yardposInfoGroup
            .selectAll('g.yard-pos-info')
            .data(this.poses, (pos, idx) => JSON.stringify(pos));
        // 更新
        cell
            .selectAll('path')
            .transition()
            .attr('stroke', (pos) => this.draw('stroke', pos))
            .attr('stroke-width', (pos) => this.draw('strokeWidth', pos))
            .attr('fill', (pos) => this._fillFunction(pos));
        cell.selectAll('text').text((pos) => this._renderPosText(pos));
        // cell
        //   .selectAll('text')
        //   .text((posInfo: Yardpos<T>) => (posInfo.text ? posInfo.text : ''));
        // 新增
        const enteredCell = cell.enter();
        this.renderYardposInfo(enteredCell);
        // 删除
        cell
            .exit()
            .transition()
            .attr('transform', (pos) => {
            const x = parseInt(this.yardposParser.getP(pos.name), 10) * this.cellSize;
            // return `translate(${x}, -${this.cellSize})`;
            return `scale(0,0)`;
        })
            .remove();
    }
    renderYardposInfo(selection) {
        const g = selection
            .append('g')
            .style('cursor', 'pointer')
            .attr('class', 'yard-pos-info')
            .attr('transform', (pos) => {
            const x = parseInt(this.yardposParser.getP(pos.name), 10) * this.cellSize;
            const y = (this.displaySize.tier - +this.yardposParser.getC(pos.name)) * this.cellSize;
            return `translate(${x}, -${y + this.cellSize})`;
        })
            .on('mouseover', function (data, i, nodes) {
            d3.select(nodes[i])
                .select('path')
                .attr('fill', 'grey');
        })
            .on('mouseleave', (data, i, nodes) => {
            d3.select(nodes[i])
                .select('path')
                .attr('fill', d => this._fillFunction(data));
        })
            .on('click', (posInfo, index) => {
            this.yardposClick.emit(posInfo);
        });
        g.append('path')
            .attr('d', (pos) => {
            // 基础图形是一个框
            const path = `M0 0 L${this.cellSize} 0 L${this.cellSize} ${this.cellSize} L0 ${this.cellSize} Z`;
            // if (data.isLocked) {
            //   // 箱区封锁要画叉
            //   path =
            //     path +
            //     `M0 0 L${this.cellSize} ${this.cellSize} M${this.cellSize} 0 L0 ${
            //       this.cellSize
            //     }`;
            // }
            return path;
        })
            .attr('fill', (pos) => this._fillFunction(pos))
            .attr('stroke', pos => this.draw('stroke', pos))
            .attr('stroke-width', pos => this.draw('strokeWidth', pos));
        // 高箱 需要加一条粗线
        // g.append('path')
        //   .attr('d', data => {
        //     if (data.container && data.container.height + '' === '9.6') {
        //       return `M0 2 L${this.cellSize} 2`;
        //     } else {
        //       return `M0 0 L${this.cellSize} 0`;
        //     }
        //   })
        //   .attr('stroke', 'black')
        //   .attr('stroke-width', data => {
        //     if (data.container && data.container.height + '' === '9.6') {
        //       return 4;
        //     } else {
        //       return 1;
        //     }
        //   });
        g.append('text')
            .attr('font-size', '9')
            .attr('text-anchor', 'middle')
            .attr('dx', this.cellSize / 2)
            .attr('dy', this.cellSize / 1.2)
            .text((pos) => this._renderPosText(pos));
        g.transition()
            // .delay((pos: Yardpos<T>) => {
            //   const tier = +this.yardposParser.getC(pos.name);
            //   return tier * 100;
            // })
            .attr('transform', (pos) => {
            const x = parseInt(this.yardposParser.getP(pos.name), 10) * this.cellSize;
            const y = (this.displaySize.tier - +this.yardposParser.getC(pos.name)) * this.cellSize;
            return `translate(${x}, ${y})`;
        });
    }
    _fillFunction(pos) {
        // 如果提供了填充色选项则使用填充色选项的配置
        if (this.renderOptions && this.renderOptions.fill) {
            if (typeof this.renderOptions.fill === 'string') {
                return this.renderOptions.fill;
            }
            else {
                return this.renderOptions.fill(pos);
            }
        }
        else {
            // 默认填充色配置
            if (pos.data) {
                return 'red';
            }
            else {
                return 'white';
            }
        }
    }
    draw(which, target) {
        if (this.renderOptions && this.renderOptions[which]) {
            switch (typeof this.renderOptions[which]) {
                case 'string':
                    return this.renderOptions[which];
                case 'number':
                    return `${this.renderOptions[which]}`;
                default:
                    return this.renderOptions[which](target);
            }
        }
        else {
            return null;
        }
    }
    _renderPosText(pos) {
        if (this.renderOptions && this.renderOptions.text) {
            if (typeof this.renderOptions.text === 'string') {
                return this.renderOptions.text;
            }
            else {
                return this.renderOptions.text(pos);
            }
        }
        else {
            return '';
        }
    }
}
CtYardBayComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.0", ngImport: i0, type: CtYardBayComponent, deps: [{ token: i0.ElementRef }, { token: i1.CtYardposParserService }], target: i0.ɵɵFactoryTarget.Component });
CtYardBayComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.3.0", type: CtYardBayComponent, selector: "ct-yard-bay", inputs: { size: "size", renderOptions: "renderOptions", cellSize: "cellSize", yardBay: "yardBay" }, outputs: { yardposClick: "yardposClick" }, usesOnChanges: true, ngImport: i0, template: "<svg class=\"yard-bay\"></svg>\n", styles: [":host{display:inline-block}\n"] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.0", ngImport: i0, type: CtYardBayComponent, decorators: [{
            type: Component,
            args: [{ selector: 'ct-yard-bay', template: "<svg class=\"yard-bay\"></svg>\n", styles: [":host{display:inline-block}\n"] }]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }, { type: i1.CtYardposParserService }]; }, propDecorators: { size: [{
                type: Input
            }], renderOptions: [{
                type: Input
            }], cellSize: [{
                type: Input
            }], yardBay: [{
                type: Input
            }], yardposClick: [{
                type: Output
            }] } });

class CtYardBayModule {
}
CtYardBayModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.0", ngImport: i0, type: CtYardBayModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
CtYardBayModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.3.0", ngImport: i0, type: CtYardBayModule, declarations: [CtYardBayComponent], imports: [CommonModule], exports: [CtYardBayComponent] });
CtYardBayModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.3.0", ngImport: i0, type: CtYardBayModule, imports: [[CommonModule]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.0", ngImport: i0, type: CtYardBayModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [CommonModule],
                    declarations: [CtYardBayComponent],
                    exports: [CtYardBayComponent]
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { CtYardBayComponent, CtYardBayModule };
//# sourceMappingURL=ng-container-terminal-yard-bay.mjs.map
