import * as i0 from '@angular/core';
import { EventEmitter, Component, ChangeDetectionStrategy, Input, Output, NgModule } from '@angular/core';
import * as d3 from 'd3';
import { CommonModule } from '@angular/common';

class CtYardOverviewComponent {
    constructor(el) {
        this.el = el;
        this.ctxWidth = 0;
        this.ctxHeight = 0;
        this.defaultRenderOptions = {
            fill: yardInfo => {
                return d3.interpolateSpectral(yardInfo.x / 3000 + yardInfo.y / 3000);
            },
            scaleFactor: 1,
            stroke: 'rgba(0, 0, 0)',
            strokeWidth: 2,
            text: yardInfo => yardInfo.block + '-' + yardInfo.direction
        };
        this._renderOptions = Object.assign({}, this.defaultRenderOptions);
        this.yardInfoList = [];
        this.yardClick = new EventEmitter();
        this.yardContentRender = new EventEmitter();
        this.host = d3.select(this.el.nativeElement);
    }
    set renderOptions(options) {
        if (options) {
            this._renderOptions = Object.assign({}, this.defaultRenderOptions, options);
        }
    }
    get renderOptions() {
        return this._renderOptions;
    }
    ngOnInit() {
        this.initYardOverview();
    }
    ngOnChanges() {
        if (!this.svgCtx) {
            this.initYardOverview();
        }
        else {
            this.redraw();
        }
    }
    initYardOverview() {
        this.ctxWidth = Math.max(...this.yardInfoList.map(d => d.x + d.width + 16), 0);
        this.ctxHeight = Math.max(...this.yardInfoList.map(d => d.y + d.height + 16), 0);
        this.host
            .select('svg')
            .selectAll('g.ct-yard-overview-group')
            .remove();
        this.svgCtx = this.host
            .select('svg')
            .attr('width', this.ctxWidth * this.renderOptions.scaleFactor + 'px')
            .attr('height', this.ctxHeight * this.renderOptions.scaleFactor + 'px');
        this.yardOverviewGroup = this.svgCtx
            .append('g')
            .attr('class', 'ct-yard-overview-group')
            .attr('transform', `scale(${this.renderOptions.scaleFactor})`)
            .attr('width', '100%')
            .attr('height', '100%');
        this.redraw();
    }
    scaleYardOverview() {
        this.svgCtx = this.host
            .select('svg')
            .attr('width', this.ctxWidth * this.renderOptions.scaleFactor + 'px')
            .attr('height', this.ctxHeight * this.renderOptions.scaleFactor + 'px');
        this.yardOverviewGroup
            .attr('transform', `scale(${this.renderOptions.scaleFactor})`)
            .attr('width', '100%')
            .attr('height', '100%');
    }
    redraw() {
        this.ctxWidth = Math.max(...this.yardInfoList.map(d => d.x + d.width + 16), 0);
        this.ctxHeight = Math.max(...this.yardInfoList.map(d => d.y + d.height + 16), 0);
        this.svgCtx
            .transition()
            .duration(500)
            .attr('width', this.ctxWidth * this.renderOptions.scaleFactor + 'px')
            .attr('height', this.ctxHeight * this.renderOptions.scaleFactor + 'px');
        this.yardOverviewGroup
            .transition()
            .duration(500)
            .attr('transform', `scale(${this.renderOptions.scaleFactor})`);
        const allYards = this.yardOverviewGroup
            .selectAll('g.yard-group')
            .data(this.yardInfoList, (pos, idx) => JSON.stringify(pos));
        // overview
        //   .selectAll('rect.yard-rect')
        //   .transition()
        //   .attr('fill', (yardInfo: YardInfo<T>) => this.draw('fill', yardInfo));
        const enterYards = allYards.enter();
        const updateYards = allYards;
        const exitYards = allYards.exit();
        const yardGroup = enterYards
            .append('g')
            .attr('class', 'yard-group')
            .attr('transform', data => {
            const x = data.x;
            const y = this.ctxHeight - data.y - data.height;
            return `translate(${x}, ${y})`;
        })
            .on('mouseover', (data, i, nodes) => {
            d3.select(nodes[i])
                .select('rect')
                .attr('fill', 'grey');
        })
            .on('mouseleave', (data, i, nodes) => {
            d3.select(nodes[i])
                .select('rect')
                .attr('fill', (yardInfo) => this.draw('fill', yardInfo));
        })
            .on('click', yardInfo => {
            this.yardClick.emit(yardInfo);
        });
        yardGroup
            .append('rect')
            .attr('class', 'yard-rect')
            .attr('width', data => data.width)
            .attr('height', data => data.height)
            .attr('fill', (yardInfo) => this.draw('fill', yardInfo))
            .attr('stroke', (yardInfo) => this.draw('stroke', yardInfo))
            .attr('stroke-width', (yardInfo) => this.draw('strokeWidth', yardInfo) + 'px');
        yardGroup.each((data, nodeIdx, nodes) => {
            this.yardContentRender.next({
                node: d3.select(nodes[nodeIdx]),
                data: data
            });
        });
        yardGroup
            .append('text')
            .attr('font-size', '12')
            .attr('text-anchor', 'middle')
            .text((yardInfo) => this.draw('text', yardInfo));
        updateYards
            .transition()
            .duration(500)
            .attr('transform', (data) => {
            const x = data.x;
            const y = this.ctxHeight - data.y - data.height;
            return `translate(${x}, ${y})`;
        });
        updateYards.each((data, nodeIdx, nodes) => {
            this.yardContentRender.next({
                node: d3.select(nodes[nodeIdx]),
                data: data
            });
        });
        // .on('mouseover', (data, i, nodes) => {
        //   d3.select(nodes[i])
        //     .select('rect')
        //     .attr('fill', 'grey');
        // })
        // .on('mouseleave', (data, i, nodes) => {
        //   d3.select(nodes[i])
        //     .select('rect')
        //     .attr('fill', (yardInfo: YardInfo<T>) => this.draw('fill', yardInfo));
        // })
        // .on('click', (data: YardInfo<T>) => {
        //   this.yardClick.emit(data);
        // });
        updateYards
            .selectAll('rect.yard-rect')
            .transition()
            .duration(500)
            .attr('width', (yardInfo) => yardInfo.width)
            .attr('height', (yardInfo) => yardInfo.height)
            .attr('fill', (yardInfo) => this.draw('fill', yardInfo))
            .attr('stroke', (yardInfo) => this.draw('stroke', yardInfo))
            .attr('stroke-width', (yardInfo) => this.draw('strokeWidth', yardInfo) + 'px');
        updateYards.selectAll('text').text((yardInfo) => this.draw('text', yardInfo));
        exitYards
            .attr('opacity', 1)
            .transition()
            .duration(500)
            .attr('opacity', 0)
            .remove();
    }
    draw(which, yardInfo) {
        if (this.renderOptions[which]) {
            switch (typeof this.renderOptions[which]) {
                case 'string':
                    return this.renderOptions[which];
                case 'number':
                    return `${this.renderOptions[which]}`;
                default:
                    return this.renderOptions[which](yardInfo);
            }
        }
        else {
            return null;
        }
    }
}
CtYardOverviewComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.0", ngImport: i0, type: CtYardOverviewComponent, deps: [{ token: i0.ElementRef }], target: i0.ɵɵFactoryTarget.Component });
CtYardOverviewComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.3.0", type: CtYardOverviewComponent, selector: "ct-yard-overview", inputs: { renderOptions: "renderOptions", yardInfoList: "yardInfoList" }, outputs: { yardClick: "yardClick", yardContentRender: "yardContentRender" }, usesOnChanges: true, ngImport: i0, template: "<svg class=\"ct-yard-overview\"></svg>\n", styles: [":host{display:block}\n"], changeDetection: i0.ChangeDetectionStrategy.OnPush });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.0", ngImport: i0, type: CtYardOverviewComponent, decorators: [{
            type: Component,
            args: [{ selector: 'ct-yard-overview', changeDetection: ChangeDetectionStrategy.OnPush, template: "<svg class=\"ct-yard-overview\"></svg>\n", styles: [":host{display:block}\n"] }]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }]; }, propDecorators: { renderOptions: [{
                type: Input
            }], yardInfoList: [{
                type: Input
            }], yardClick: [{
                type: Output
            }], yardContentRender: [{
                type: Output
            }] } });

class CtYardOverviewModule {
}
CtYardOverviewModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.0", ngImport: i0, type: CtYardOverviewModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
CtYardOverviewModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.3.0", ngImport: i0, type: CtYardOverviewModule, declarations: [CtYardOverviewComponent], imports: [CommonModule], exports: [CtYardOverviewComponent] });
CtYardOverviewModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.3.0", ngImport: i0, type: CtYardOverviewModule, imports: [[CommonModule]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.0", ngImport: i0, type: CtYardOverviewModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [CommonModule],
                    declarations: [CtYardOverviewComponent],
                    exports: [CtYardOverviewComponent]
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { CtYardOverviewComponent, CtYardOverviewModule };
//# sourceMappingURL=ng-container-terminal-yard-overview.mjs.map
