import * as i0 from '@angular/core';
import { InjectionToken, Injectable, Optional, Inject } from '@angular/core';

const VESCELL_PARSER_CONFIG = new InjectionToken('vescellParserConfig');

const YARDPOS_PARSER_CONFIG = new InjectionToken('yardposParserConfig');

class CtYardposParserService {
    constructor(config) {
        this.config = config;
        this.defaultConfig = {
            pattern: 'QQQWWWPPCC'
        };
        this.idxMap = {
            Q: { s: 0, e: 3 },
            W: { s: 3, e: 5 },
            P: { s: 5, e: 7 },
            C: { s: 7, e: 8 }
        };
        // 合并用户提供的解析器配置
        this.finalConfig = Object.assign({}, this.defaultConfig, config);
        // 重新计算索引
        this._updateIdxMap();
    }
    _updateIdxMap() {
        this.idxMap['Q']['s'] = this.finalConfig.pattern.indexOf('Q');
        this.idxMap['Q']['e'] = this.finalConfig.pattern.lastIndexOf('Q') + 1;
        this.idxMap['W']['s'] = this.finalConfig.pattern.indexOf('W');
        this.idxMap['W']['e'] = this.finalConfig.pattern.lastIndexOf('W') + 1;
        this.idxMap['P']['s'] = this.finalConfig.pattern.indexOf('P');
        this.idxMap['P']['e'] = this.finalConfig.pattern.lastIndexOf('P') + 1;
        this.idxMap['C']['s'] = this.finalConfig.pattern.indexOf('C');
        this.idxMap['C']['e'] = this.finalConfig.pattern.lastIndexOf('C') + 1;
    }
    _padLeft(num, length, text = '0') {
        return (Array(length).join(text) + num).slice(-length);
    }
    getQ(yardpos) {
        return yardpos.slice(this.idxMap['Q']['s'], this.idxMap['Q']['e']);
    }
    getW(yardpos) {
        return yardpos.slice(this.idxMap['W']['s'], this.idxMap['W']['e']);
    }
    getP(yardpos) {
        return yardpos.slice(this.idxMap['P']['s'], this.idxMap['P']['e']);
    }
    getC(yardpos) {
        return yardpos.slice(this.idxMap['C']['s'], this.idxMap['C']['e']);
    }
    getQW(yardpos) {
        return this.getQ(yardpos) + this.getW(yardpos);
    }
    getQWP(yardpos) {
        return this.getQW(yardpos) + this.getP(yardpos);
    }
    formatW(bay) {
        const bayStringWidth = this.idxMap['W']['e'] - this.idxMap['W']['s'];
        if (bay.toString().length > bayStringWidth) {
            throw Error('传入参数有误，其长度应<=' + bayStringWidth);
        }
        return this._padLeft(bay.toString(), bayStringWidth, '0');
    }
    formatP(row) {
        const rowStringWidth = this.idxMap['P']['e'] - this.idxMap['P']['s'];
        if (row.toString().length > rowStringWidth) {
            throw Error('传入参数有误，其长度应<=' + rowStringWidth);
        }
        return this._padLeft(row.toString(), rowStringWidth, '0');
    }
    formatC(tier) {
        const tierStringWidth = this.idxMap['C']['e'] - this.idxMap['C']['s'];
        if (tier.toString().length > tierStringWidth) {
            throw Error('传入参数有误，其长度应<=' + tierStringWidth);
        }
        return this._padLeft(tier.toString(), tierStringWidth, '0');
    }
}
CtYardposParserService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.0", ngImport: i0, type: CtYardposParserService, deps: [{ token: YARDPOS_PARSER_CONFIG, optional: true }], target: i0.ɵɵFactoryTarget.Injectable });
CtYardposParserService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.3.0", ngImport: i0, type: CtYardposParserService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.0", ngImport: i0, type: CtYardposParserService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return [{ type: undefined, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [YARDPOS_PARSER_CONFIG]
                }] }]; } });

class CtVescellParserService {
    constructor(config) {
        this.config = config;
        this.defaultConfig = {
            pattern: 'BBBBLLCC'
        };
        this.idxMap = {
            B: { s: 0, e: 4 },
            L: { s: 4, e: 6 },
            C: { s: 6, e: 8 }
        };
        // 合并用户提供的解析器配置
        this.finalConfig = Object.assign({}, this.defaultConfig, config);
        // 重新计算索引
        this._updateIdxMap();
    }
    _updateIdxMap() {
        this.idxMap['B']['s'] = this.finalConfig.pattern.indexOf('B');
        this.idxMap['B']['e'] = this.finalConfig.pattern.lastIndexOf('B') + 1;
        this.idxMap['L']['s'] = this.finalConfig.pattern.indexOf('L');
        this.idxMap['L']['e'] = this.finalConfig.pattern.lastIndexOf('L') + 1;
        this.idxMap['C']['s'] = this.finalConfig.pattern.indexOf('C');
        this.idxMap['C']['e'] = this.finalConfig.pattern.lastIndexOf('C') + 1;
    }
    _padLeft(num, length, text = '0') {
        return (Array(length).join(text) + num).slice(-length);
    }
    getBay(vescell) {
        return vescell.slice(this.idxMap['B']['s'], this.idxMap['B']['e']);
    }
    getBayno(vescell) {
        return vescell.slice(this.idxMap['B']['s'], this.idxMap['B']['e'] - 1);
    }
    getL(vescell) {
        return vescell.slice(this.idxMap['L']['s'], this.idxMap['L']['e']);
    }
    getC(vescell) {
        return vescell.slice(this.idxMap['C']['s'], this.idxMap['C']['e']);
    }
    getLC(vescell) {
        return this.getL(vescell) + this.getC(vescell);
    }
    isDeck(vescell) {
        const bay = this.getBay(vescell);
        return bay[bay.length - 1] === 'D';
    }
    // formatW(bay: number | string) {
    //   const bayStringWidth = this.idxMap['W']['e'] - this.idxMap['W']['s'];
    //   if (bay.toString().length > bayStringWidth) {
    //     throw Error('传入参数有误，其长度应<=' + bayStringWidth);
    //   }
    //   return this._padLeft(bay.toString(), bayStringWidth, '0');
    // }
    formatP(row) {
        const rowStringWidth = this.idxMap['P']['e'] - this.idxMap['P']['s'];
        if (row.toString().length > rowStringWidth) {
            throw Error('传入参数有误，其长度应<=' + rowStringWidth);
        }
        return this._padLeft(row.toString(), rowStringWidth, '0');
    }
    formatC(tier) {
        const tierStringWidth = this.idxMap['C']['e'] - this.idxMap['C']['s'];
        if (tier.toString().length > tierStringWidth) {
            throw Error('传入参数有误，其长度应<=' + tierStringWidth);
        }
        return this._padLeft(tier.toString(), tierStringWidth, '0');
    }
}
CtVescellParserService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.0", ngImport: i0, type: CtVescellParserService, deps: [{ token: VESCELL_PARSER_CONFIG, optional: true }], target: i0.ɵɵFactoryTarget.Injectable });
CtVescellParserService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.3.0", ngImport: i0, type: CtVescellParserService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.0", ngImport: i0, type: CtVescellParserService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return [{ type: undefined, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [VESCELL_PARSER_CONFIG]
                }] }]; } });

// export * from './lib/tool/index';
// export * from './tool.module';

/**
 * Generated bundle index. Do not edit.
 */

export { CtVescellParserService, CtYardposParserService, VESCELL_PARSER_CONFIG, YARDPOS_PARSER_CONFIG };
//# sourceMappingURL=ng-container-terminal-tool.mjs.map
