import * as i0 from '@angular/core';
import { EventEmitter, Component, Input, Output, NgModule } from '@angular/core';
import * as d3 from 'd3';
import { Subject } from 'rxjs';
import { debounceTime } from 'rxjs/operators';
import * as i1 from 'ng-container-terminal/tool';
import { CommonModule } from '@angular/common';

// import { YardposInfo } from '../../model/yardpos-info';
class CtVesselBayComponent {
    constructor(el, cellParser) {
        this.el = el;
        this.cellParser = cellParser;
        this.vescellClick = new EventEmitter();
        this.contentRender = new EventEmitter();
        this.renderUpdateSubject = new Subject();
        this._vescells = [];
        this.padding = 16;
        this.deckRowLabels = [];
        this.deckTierLabels = [];
        this.deckCells = [];
        this.holdRowLabels = [];
        this.holdTierLabels = [];
        this.holdCells = [];
        this.layout = {
            deckRowCount: 0,
            deckTierCount: 0,
            deckMinTierLabel: 82,
            deckHasZeroRow: false,
            holdRowCount: 0,
            holdTierCount: 0,
            holdHasZeroRow: false,
            maxRow: 0 // 整贝最大列宽 max(deckMaxRow, holdMaxRow)
        };
        this._cellSize = 20;
        this.cellTrackByFn = (cell) => {
            return JSON.stringify(cell);
        };
        this.renderUpdateSubject.pipe(debounceTime(100)).subscribe(() => {
            this.premarshalling(this.vescells);
            this.renderAll();
        });
    }
    set renderOptions(options) {
        this._renderOptions = options;
        this.renderUpdateSubject.next();
    }
    get renderOptions() {
        return this._renderOptions;
    }
    set cellSize(size) {
        this._cellSize = +size;
        this.renderUpdateSubject.next();
    }
    get cellSize() {
        return this._cellSize;
    }
    set vescells(vescells) {
        // console.log('update vescells');
        this._vescells = vescells;
        this.renderUpdateSubject.next();
    }
    get vescells() {
        return this._vescells;
    }
    set name(name) {
        this._name = name;
        this.displayName = name;
    }
    get name() {
        return this._name;
    }
    ngOnInit() {
        // console.log('ng on init');
        this.host = d3.select(this.el.nativeElement);
        // step1
        this.svg = this.host
            .select('svg')
            .attr('width', (this.layout.maxRow + 1) * this.cellSize + 2 * this.padding)
            .attr('height', (this.layout.deckTierCount + this.layout.holdTierCount + 3) * this.cellSize + 2 * this.padding);
        this.svg.selectAll('g').remove();
        this.vesselDeckBayGroup = this.svg
            .append('g')
            .attr('class', 'vessel-deck-bay-group')
            .attr('transform', `translate(${this.padding}, ${this.padding})`);
        this.vesselHoldBayGroup = this.svg
            .append('g')
            .attr('class', 'vessel-hold-bay-group')
            .attr('transform', (label, index) => {
            const x = this.padding;
            const y = this.cellSize * (this.layout.deckTierCount + 2) + this.padding; // +2 为2组列标签占位
            return `translate(${x}, ${y})`;
        });
        // deck tier label
        this.vesselDeckBayGroup.selectAll('g.vessel-deck-bay-tier-label-group').remove();
        this.vesselDeckBayTierLabelGroup = this.vesselDeckBayGroup
            .append('g')
            .attr('class', 'vessel-deck-bay-tier-label-group');
        // 绘制甲板列标签deck row label
        this.vesselDeckBayGroup.selectAll('g.vessel-deck-bay-row-label-group').remove();
        this.vesselDeckBayRowLabelGroup = this.vesselDeckBayGroup
            .append('g')
            .attr('class', 'vessel-deck-bay-row-label-group');
        // hold tier label
        this.vesselHoldBayGroup.selectAll('g.vessel-hold-bay-tier-label-group').remove();
        this.vesselHoldBayTierLabelGroup = this.vesselHoldBayGroup
            .append('g')
            .attr('class', 'vessel-hold-bay-tier-label-group');
        // hold row label
        this.vesselHoldBayGroup.selectAll('g.vessel-hold-bay-row-label-group').remove();
        this.vesselHoldBayRowLabelGroup = this.vesselHoldBayGroup
            .append('g')
            .attr('class', 'vessel-hold-bay-row-label-group');
        // step3 划定舱内舱面船箱位vescell绘制区域
        this.vesselDeckBayGroup.selectAll('g.vessel-deck-bay-cell-group').remove();
        this.vesselHoldBayGroup.selectAll('g.vessel-hold-bay-cell-group').remove();
        this.vesselDeckBayCellGroup = this.vesselDeckBayGroup.append('g').attr('class', 'vessel-deck-bay-cell-group');
        this.vesselHoldBayCellGroup = this.vesselHoldBayGroup.append('g').attr('class', 'vessel-hold-bay-cell-group');
    }
    /**
     * 船箱位数据发生变化时，重新对输入数据进行预处理
     *
     */
    premarshalling(cells) {
        if (!cells || cells.length <= 0) {
            return;
        }
        if (!this.name) {
            this.displayName = this.cellParser.getBayno(cells[0].name);
        }
        const deckRows = new Set();
        const holdRows = new Set();
        const deckTiers = new Set();
        const holdTiers = new Set();
        const deckCells = [];
        const holdCells = [];
        cells.forEach(cell => {
            if (this.cellParser.isDeck(cell.name)) {
                deckCells.push(cell);
                deckRows.add(this.cellParser.getL(cell.name));
                deckTiers.add(this.cellParser.getC(cell.name));
            }
            else {
                holdCells.push(cell);
                holdRows.add(this.cellParser.getL(cell.name));
                holdTiers.add(this.cellParser.getC(cell.name));
            }
        });
        this.deckTierLabels = Array.from(deckTiers).sort((a, b) => {
            const x = +a;
            const y = +b;
            return y - x;
        });
        // this.deckTierLabels = this.deckTierLabels.sort().reverse();
        this.holdTierLabels = Array.from(holdTiers).sort((a, b) => {
            const x = +a;
            const y = +b;
            return y - x;
        });
        this.deckRowLabels = Array.from(deckRows).sort((a, b) => {
            let x = +a;
            let y = +b;
            x = x * ((x % 2) * 2 - 1);
            y = y * ((y % 2) * 2 - 1);
            return x - y;
        });
        this.holdRowLabels = Array.from(holdRows).sort((a, b) => {
            let x = +a;
            let y = +b;
            x = x * ((x % 2) * 2 - 1);
            y = y * ((y % 2) * 2 - 1);
            return x - y;
        });
        // console.log((+deckTiers[0] - 82) / 2);
        const deckMinTierLabel = +this.deckTierLabels[this.deckTierLabels.length - 1];
        this.layout = {
            deckRowCount: deckRows.size,
            deckTierCount: this.deckTierLabels.length > 0 ? (+this.deckTierLabels[0] - deckMinTierLabel) / 2 + 1 : 0,
            deckHasZeroRow: this.hasZeroRow(deckCells),
            deckMinTierLabel: deckMinTierLabel,
            holdRowCount: holdRows.size,
            holdTierCount: this.holdTierLabels.length > 0 ? +this.holdTierLabels[0] / 2 : 0,
            holdHasZeroRow: this.hasZeroRow(holdCells),
            maxRow: Math.max(deckRows.size, holdRows.size)
        };
        this.deckCells = deckCells;
        this.holdCells = holdCells;
    }
    /**
     * ## 绘制贝位结构
     *
     * 1. 规划舱内舱面绘制区域
     * 2. 规划舱内舱面行、列标签绘制区域
     * 3. 规划舱内舱面船箱位绘制区域
     *
     */
    renderAll() {
        // step1
        this.svg
            .transition()
            .attr('width', (this.layout.maxRow + 1) * this.cellSize + 2 * this.padding)
            .attr('height', (this.layout.deckTierCount + this.layout.holdTierCount + 3) * this.cellSize + 2 * this.padding);
        this.vesselDeckBayGroup.transition().attr('transform', `translate(${this.padding}, ${this.padding})`);
        this.vesselHoldBayGroup.transition().attr('transform', (label, index) => {
            const x = this.padding;
            const y = this.cellSize * (this.layout.deckTierCount + 2) + this.padding; // +2 为2组列标签占位
            return `translate(${x}, ${y})`;
        });
        // step 2
        // 绘甲板制层标签deck tier
        //        bayno
        //  1 2 3 4 5 6 7 8
        // 1
        // 2
        // 3
        const deckRowCount = this.deckRowLabels.length;
        const holdRowCount = this.holdRowLabels.length;
        let holdRowOffset = 0;
        let deckRowOffset = 0;
        if (deckRowCount >= holdRowCount) {
            holdRowOffset = ((deckRowCount - holdRowCount) / 2) * this.cellSize;
        }
        else {
            deckRowOffset = ((holdRowCount - deckRowCount) / 2) * this.cellSize;
        }
        this.renderTierLabels(this.vesselDeckBayTierLabelGroup, this.deckTierLabels);
        // 绘制甲板列标签deck row label
        this.renderRowLabels(this.vesselDeckBayRowLabelGroup, this.deckRowLabels, deckRowOffset);
        // *******HOLD********
        // 舱内层标签
        this.renderTierLabels(this.vesselHoldBayTierLabelGroup, this.holdTierLabels);
        // 舱内列标签
        this.renderRowLabels(this.vesselHoldBayRowLabelGroup, this.holdRowLabels, holdRowOffset);
        // step3 划定舱内舱面船箱位vescell绘制区域
        this.vesselDeckBayCellGroup
            // .transition()
            .attr('transform', (label, index) => {
            const x = deckRowOffset + this.cellSize;
            const y = this.cellSize; // 舱面列标签占位
            return `translate(${x}, ${y})`;
        });
        this.vesselHoldBayCellGroup
            // .transition()
            .attr('transform', (label, index) => {
            const x = holdRowOffset + this.cellSize;
            const y = this.cellSize; // 舱内列标签占位
            return `translate(${x}, ${y})`;
        });
        this.renderVescells();
    }
    renderTierLabels(selection, data) {
        const tierLabelSelection = selection.selectAll('text').data(data, (label) => label);
        const enteredTierLabels = tierLabelSelection.enter();
        const updatedTierLabels = tierLabelSelection;
        const exitedTierLabels = tierLabelSelection.exit();
        // console.log(enteredTierLabels, updatedTierLabels, exitedTierLabels);
        enteredTierLabels
            .append('text')
            .attr('transform', (_, index) => {
            const x = -this.cellSize;
            const y = (index + 1) * this.cellSize;
            return `translate(${x}, ${y})`;
        })
            .attr('opacity', 0)
            .transition()
            .duration(500)
            .attr('opacity', 1)
            .attr('width', this.cellSize)
            .attr('height', this.cellSize)
            .attr('transform', (label, index) => {
            const x = 0;
            const y = (index + 1) * this.cellSize;
            return `translate(${x}, ${y})`;
        })
            .attr('font-size', '12')
            .attr('text-anchor', 'start')
            // .attr('dx', this.cellSize / 2)
            .attr('dy', this.cellSize / 2 + 12 / 2)
            .text(c => c);
        updatedTierLabels
            .transition()
            .attr('opacity', 1)
            .attr('width', this.cellSize)
            .attr('height', this.cellSize)
            .attr('transform', (label, index) => {
            const x = 0;
            const y = (index + 1) * this.cellSize;
            return `translate(${x}, ${y})`;
        })
            .attr('font-size', '12')
            .attr('text-anchor', 'start')
            // .attr('dx', this.cellSize / 2)
            .attr('dy', this.cellSize / 2 + 12 / 2)
            .text((c) => c);
        exitedTierLabels.remove();
    }
    renderRowLabels(selection, data, rowOffset) {
        const rowLabelSelection = selection.selectAll('text').data(data, (label) => label);
        const enteredRowLabels = rowLabelSelection.enter();
        const updatedRowLabels = rowLabelSelection;
        const exitedRowLabels = rowLabelSelection.exit();
        // console.log('row labels', updatedRowLabels);
        enteredRowLabels
            .append('text')
            .attr('transform', (_, index) => {
            const x = rowOffset + (index + 1) * this.cellSize;
            const y = -this.cellSize;
            return `translate(${x}, ${y})`;
        })
            .attr('opacity', 0)
            .transition()
            .duration(500)
            .attr('width', this.cellSize)
            .attr('height', this.cellSize)
            .attr('transform', (_, index) => {
            const x = rowOffset + (index + 1) * this.cellSize;
            const y = 0;
            return `translate(${x}, ${y})`;
        })
            .attr('opacity', 1)
            .attr('font-size', '12')
            .attr('text-anchor', 'middle')
            .attr('dx', this.cellSize / 2)
            .attr('dy', this.cellSize / 2)
            .text(c => c);
        updatedRowLabels
            .transition()
            .duration(500)
            .attr('transform', (_, index) => {
            const x = rowOffset + (index + 1) * this.cellSize;
            const y = 0;
            return `translate(${x}, ${y})`;
        })
            .attr('width', this.cellSize)
            .attr('height', this.cellSize)
            .attr('opacity', 1)
            .attr('font-size', '12')
            .attr('text-anchor', 'middle')
            .attr('dx', this.cellSize / 2)
            .attr('dy', this.cellSize / 2)
            .text((c) => c);
        exitedRowLabels
            .transition()
            .duration(500)
            .attr('transform', (label, index) => {
            const x = rowOffset + (index + 1) * this.cellSize;
            const y = -12;
            return `translate(${x}, ${y})`;
        })
            .attr('opacity', 0);
    }
    renderVescells() {
        const deckCells = this.vesselDeckBayCellGroup
            .selectAll('g')
            .data(this.deckCells, (cell) => this.cellTrackByFn(cell));
        const enteredDeckCells = deckCells.enter();
        const updatedDeckCells = deckCells;
        const exitedDeckCells = deckCells.exit();
        // console.log('deck cells', updatedDeckCells);
        const enteredDeckG = enteredDeckCells
            .append('g')
            .attr('transform', cell => {
            const row = +this.cellParser.getL(cell.name);
            const tier = (+this.cellParser.getC(cell.name) - this.layout.deckMinTierLabel) / 2;
            let x = row * ((row % 2) * 2 - 1);
            if (this.layout.deckHasZeroRow) {
                x = x + (row % 2);
            }
            else {
                x = x - (row % 2);
            }
            x = x / 2 + (this.layout.deckRowCount - +this.layout.deckHasZeroRow) / 2;
            const y = (this.layout.deckTierCount - tier - 1) * this.cellSize;
            x = this.cellSize * x;
            return `translate(${x}, ${y})`;
        })
            .on('click', (cell, index) => {
            this.vescellClick.emit(cell);
            // console.log('vescell click', cell);
        });
        enteredDeckG
            // .selectAll('rect')
            .append('rect')
            .attr('stroke-width', '2px')
            .attr('stroke', 'black')
            .attr('fill', 'white')
            // .transition()
            // .duration(500)
            .attr('width', this.cellSize)
            .attr('height', this.cellSize)
            .attr('fill', (cell) => this._fillFunction(cell));
        enteredDeckG
            // .selectAll('text')
            .append('text')
            .attr('font-size', '9')
            .attr('text-anchor', 'middle')
            .attr('dx', this.cellSize / 2)
            .attr('dy', this.cellSize / 1.2)
            .text((cell) => this._renderText(cell));
        enteredDeckG.each((data, nodeIdx, nodes) => {
            this.contentRender.next({
                node: d3.select(nodes[nodeIdx]),
                data: data
            });
        });
        updatedDeckCells
            .transition()
            .duration(500)
            .attr('transform', cell => {
            const row = +this.cellParser.getL(cell.name);
            const tier = (+this.cellParser.getC(cell.name) - this.layout.deckMinTierLabel) / 2;
            let x = row * ((row % 2) * 2 - 1);
            if (this.layout.deckHasZeroRow) {
                x = x + (row % 2);
            }
            else {
                x = x - (row % 2);
            }
            x = x / 2 + (this.layout.deckRowCount - +this.layout.deckHasZeroRow) / 2;
            const y = (this.layout.deckTierCount - tier - 1) * this.cellSize;
            x = this.cellSize * x;
            return `translate(${x}, ${y})`;
        });
        updatedDeckCells
            .selectAll('rect')
            .attr('width', this.cellSize)
            .attr('height', this.cellSize)
            .attr('fill', (cell) => this._fillFunction(cell));
        updatedDeckCells
            .selectAll('text')
            .attr('dx', this.cellSize / 2)
            .attr('dy', this.cellSize / 1.2)
            .text((cell) => this._renderText(cell));
        updatedDeckCells.each((data, nodeIdx, nodes) => {
            this.contentRender.next({
                node: d3.select(nodes[nodeIdx]),
                data: data
            });
        });
        exitedDeckCells
            .transition()
            .attr('transform', (pos) => {
            return `scale(0,0)`;
        })
            .remove();
        // hold cell group
        const holdCells = this.vesselHoldBayCellGroup
            .selectAll('g')
            .data(this.holdCells, (cell) => this.cellTrackByFn(cell));
        const enteredHoldCells = holdCells.enter();
        const updatedHoldCells = holdCells;
        const exitedHoldCells = holdCells.exit();
        const enteredHoldG = enteredHoldCells
            .append('g')
            .attr('transform', cell => {
            const row = +this.cellParser.getL(cell.name);
            const tier = +this.cellParser.getC(cell.name) / 2;
            let x = row * ((row % 2) * 2 - 1);
            if (this.layout.holdHasZeroRow) {
                x = x + (row % 2);
            }
            else {
                x = x - (row % 2);
            }
            x = x / 2 + (this.layout.holdRowCount - +this.layout.holdHasZeroRow) / 2;
            const y = (this.layout.holdTierCount - tier) * this.cellSize;
            x = this.cellSize * x;
            return `translate(${x}, ${y})`;
        })
            .on('click', (cell, index) => {
            this.vescellClick.emit(cell);
            // console.log('vescell click', cell);
        });
        enteredHoldG
            .append('rect')
            .attr('stroke-width', '2px')
            .attr('stroke', 'black')
            .attr('fill', 'white')
            .transition()
            .duration(500)
            .attr('width', this.cellSize)
            .attr('height', this.cellSize)
            .attr('fill', (cell) => this._fillFunction(cell));
        enteredHoldG
            .append('text')
            .attr('font-size', '9')
            .attr('text-anchor', 'middle')
            .attr('dx', this.cellSize / 2)
            .attr('dy', this.cellSize / 1.2)
            .text((cell) => this._renderText(cell));
        enteredHoldG.each((data, nodeIdx, nodes) => {
            this.contentRender.next({
                node: d3.select(nodes[nodeIdx]),
                data: data
            });
        });
        updatedHoldCells
            .transition()
            .duration(500)
            .attr('transform', cell => {
            const row = +this.cellParser.getL(cell.name);
            const tier = +this.cellParser.getC(cell.name) / 2;
            let x = row * ((row % 2) * 2 - 1);
            if (this.layout.holdHasZeroRow) {
                x = x + (row % 2);
            }
            else {
                x = x - (row % 2);
            }
            x = x / 2 + (this.layout.holdRowCount - +this.layout.holdHasZeroRow) / 2;
            const y = (this.layout.holdTierCount - tier) * this.cellSize;
            x = this.cellSize * x;
            return `translate(${x}, ${y})`;
        });
        // .attr('width', this.cellSize)
        // .attr('height', this.cellSize)
        // .attr('fill', (cell: Vescell<T>) => this._fillFunction(cell));
        updatedHoldCells
            .selectAll('rect')
            .attr('width', this.cellSize)
            .attr('height', this.cellSize)
            .attr('fill', (cell) => this._fillFunction(cell));
        updatedHoldCells
            .selectAll('text')
            .attr('dx', this.cellSize / 2)
            .attr('dy', this.cellSize / 1.2)
            .text((cell) => this._renderText(cell));
        updatedHoldCells.each((data, nodeIdx, nodes) => {
            this.contentRender.next({
                node: d3.select(nodes[nodeIdx]),
                data: data
            });
        });
        exitedHoldCells
            .transition()
            .attr('transform', (pos) => {
            return `scale(0,0)`;
        })
            .remove();
    }
    _fillFunction(cell) {
        // 如果提供了填充色选项则使用填充色选项的配置
        if (this.renderOptions && this.renderOptions.fill) {
            if (typeof this.renderOptions.fill === 'string') {
                return this.renderOptions.fill;
            }
            else {
                return this.renderOptions.fill(cell);
            }
        }
        else {
            // 默认填充色配置
            if (cell.data) {
                return 'red';
            }
            else {
                return 'white';
            }
        }
    }
    hasZeroRow(cells) {
        for (let index = 0; index < cells.length; index++) {
            const cell = cells[index];
            if (+this.cellParser.getL(cell.name) === 0) {
                return true;
            }
        }
        return false;
    }
    _renderText(cell) {
        if (this.renderOptions && this.renderOptions.text) {
            if (typeof this.renderOptions.text === 'string') {
                return this.renderOptions.text;
            }
            else {
                return this.renderOptions.text(cell);
            }
        }
        else {
            return '';
        }
    }
}
CtVesselBayComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.0", ngImport: i0, type: CtVesselBayComponent, deps: [{ token: i0.ElementRef }, { token: i1.CtVescellParserService }], target: i0.ɵɵFactoryTarget.Component });
CtVesselBayComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.3.0", type: CtVesselBayComponent, selector: "ct-vessel-bay", inputs: { renderOptions: "renderOptions", cellSize: "cellSize", vescells: "vescells", name: "name", cellTrackByFn: "cellTrackByFn" }, outputs: { vescellClick: "vescellClick", contentRender: "contentRender" }, ngImport: i0, template: "<div class=\"bay-label\">{{displayName}}</div>\n<svg class=\"ct-vescell-bay\"></svg>\n", styles: [":host{display:inline-block}.bay-label{text-align:center}\n"] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.0", ngImport: i0, type: CtVesselBayComponent, decorators: [{
            type: Component,
            args: [{ selector: 'ct-vessel-bay', template: "<div class=\"bay-label\">{{displayName}}</div>\n<svg class=\"ct-vescell-bay\"></svg>\n", styles: [":host{display:inline-block}.bay-label{text-align:center}\n"] }]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }, { type: i1.CtVescellParserService }]; }, propDecorators: { renderOptions: [{
                type: Input
            }], cellSize: [{
                type: Input
            }], vescells: [{
                type: Input
            }], name: [{
                type: Input
            }], vescellClick: [{
                type: Output
            }], contentRender: [{
                type: Output
            }], cellTrackByFn: [{
                type: Input
            }] } });

class CtVesselBayModule {
}
CtVesselBayModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.0", ngImport: i0, type: CtVesselBayModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
CtVesselBayModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.3.0", ngImport: i0, type: CtVesselBayModule, declarations: [CtVesselBayComponent], imports: [CommonModule], exports: [CtVesselBayComponent] });
CtVesselBayModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.3.0", ngImport: i0, type: CtVesselBayModule, imports: [[CommonModule]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.0", ngImport: i0, type: CtVesselBayModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [CommonModule],
                    declarations: [CtVesselBayComponent],
                    exports: [CtVesselBayComponent]
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { CtVesselBayComponent, CtVesselBayModule };
//# sourceMappingURL=ng-container-terminal-vessel-bay.mjs.map
